@@ -11,7 +11,7 @@ import dataclasses
 import fractions
 
 from .. import model
-from ..model import actions, expressions, types, effects, properties, operators
+from ..model import actions, expressions, types, properties, operators
 
 from . import lexer
 
@@ -176,10 +176,10 @@ _BINARY_CONSTRUCTORS: t.Mapping[lexer.TokenType, expressions.BinaryConstructor]
     lexer.TokenType.MINUS: expressions.sub,
     lexer.TokenType.COMP_EQ: expressions.equals,
     lexer.TokenType.COMP_NEQ: expressions.not_equals,
-    lexer.TokenType.COMP_LT: expressions.less_than,
-    lexer.TokenType.COMP_LE: expressions.less_or_equal_than,
-    lexer.TokenType.COMP_GE: expressions.greater_than,
-    lexer.TokenType.COMP_GT: expressions.greater_or_equal_than,
+    lexer.TokenType.COMP_LT: expressions.less,
+    lexer.TokenType.COMP_LE: expressions.less_or_equal,
+    lexer.TokenType.COMP_GE: expressions.greater_or_equal,
+    lexer.TokenType.COMP_GT: expressions.greater,
     lexer.TokenType.LOGIC_AND: expressions.logic_and,
     lexer.TokenType.LOGIC_OR: expressions.logic_or,
     lexer.TokenType.LOGIC_XOR: expressions.logic_xor,
@@ -189,7 +189,7 @@ _BINARY_CONSTRUCTORS: t.Mapping[lexer.TokenType, expressions.BinaryConstructor]
 
 
 _BuiltinExpressionConstructor = t.Callable[[t.List[model.Expression]], model.Expression]
-_BuiltinPropertyConstructor = t.Callable[[t.List[model.Property]], model.Property]
+_BuiltinPropertyConstructor = t.Callable[[t.List[model.Expression]], model.Expression]
 
 
 @d.dataclass
@@ -241,35 +241,35 @@ def _construct_max(arguments: t.List[model.Expression]) -> model.Expression:
     return expressions.maximum(arguments[0], arguments[1])
 
 
-def _construct_probability_min(arguments: t.List[model.Property]) -> model.Property:
+def _construct_probability_min(arguments: t.List[model.Expression]) -> model.Expression:
     if len(arguments) != 1:
         raise Exception(f"Pmin takes exactly 1 argument but {len(arguments)} are given")
     return properties.min_prob(arguments[0])
 
 
-def _construct_probability_max(arguments: t.List[model.Property]) -> model.Property:
+def _construct_probability_max(arguments: t.List[model.Expression]) -> model.Expression:
     if len(arguments) != 1:
         raise Exception(f"Pmax takes exactly 1 argument but {len(arguments)} are given")
     return properties.max_prob(arguments[0])
 
 
 def _construct_finally(
-    arguments: t.List[model.Property],
-) -> model.Property:
+    arguments: t.List[model.Expression],
+) -> model.Expression:
     if len(arguments) != 1:
         raise Exception(f"F takes exactly 1 argument but {len(arguments)} are given")
     return properties.eventually(arguments[0])
 
 
 def _construct_globally(
-    arguments: t.List[model.Property],
-) -> model.Property:
+    arguments: t.List[model.Expression],
+) -> model.Expression:
     if len(arguments) != 1:
         raise Exception(f"G takes exactly 1 argument but {len(arguments)} are given")
     return properties.globally(arguments[0])
 
 
-_BUILTIN_FUNCTIONS: t.Mapping[str, _BuiltinExpressionConstructor] = {
+_BUILTIN_FUNCTIONS: t.Dict[str, _BuiltinExpressionConstructor] = {
     "floor": _construct_floor,
     "ceil": _construct_ceil,
     "abs": _construct_abs,
@@ -279,6 +279,30 @@ _BUILTIN_FUNCTIONS: t.Mapping[str, _BuiltinExpressionConstructor] = {
     "max": _construct_max,
 }
 
+
+def _update_builtin_functions() -> None:
+    for trigonometric_function in operators.TrigonometricFunction:
+
+        def _make_constructor(
+            function: operators.TrigonometricFunction,
+        ) -> _BuiltinExpressionConstructor:
+            def construct(arguments: t.List[model.Expression]) -> model.Expression:
+                if len(arguments) != 1:
+                    symbol = function.symbol
+                    raise Exception(
+                        f"{symbol} takes exactly 1 argument but {len(arguments)} are given"
+                    )
+                return expressions.Trigonometric(function, arguments[0])
+
+            return construct
+
+        construct = _make_constructor(trigonometric_function)
+        _BUILTIN_FUNCTIONS[trigonometric_function.symbol] = construct
+
+
+_update_builtin_functions()
+
+
 _PROPERTY_FUNCTIONS: t.Mapping[str, _BuiltinPropertyConstructor] = {
     "Pmin": _construct_probability_min,
     "Pmax": _construct_probability_max,
@@ -297,8 +321,8 @@ def _parse_property_function(
     constructor: _BuiltinPropertyConstructor,
     stream: TokenStream,
     environment: Environment,
-) -> model.Property:
-    arguments: t.List[model.Property] = []
+) -> model.Expression:
+    arguments: t.List[model.Expression] = []
     if not stream.accept(lexer.TokenType.RIGHT_PAR):
         arguments.append(parse_property(stream, environment=environment))
         while not stream.accept(lexer.TokenType.RIGHT_PAR):
@@ -321,7 +345,7 @@ def _parse_builtin_function(
     return constructor(arguments)
 
 
-def _parse_primary(stream: TokenStream, environment: Environment) -> model.Property:
+def _parse_primary(stream: TokenStream, environment: Environment) -> model.Expression:
     if stream.accept(lexer.TokenType.INTEGER, consume=False):
         return expressions.ensure_expr(int(stream.consume().text))
     elif stream.accept(lexer.TokenType.REAL, consume=False):
@@ -383,7 +407,7 @@ def _parse_primary(stream: TokenStream, environment: Environment) -> model.Prope
         raise stream.make_error("expected primary expression or property")
 
 
-def _parse_unary(stream: TokenStream, environment: Environment) -> model.Property:
+def _parse_unary(stream: TokenStream, environment: Environment) -> model.Expression:
     if stream.accept(lexer.TokenType.LOGIC_NOT):
         operand = _parse_unary(stream, environment=environment)
         # FIXME: proper error reporting
@@ -405,7 +429,7 @@ def parse_property(
     *,
     min_precedence: int = 0,
     environment: Environment = DEFAULT_ENVIRONMENT,
-) -> model.Property:
+) -> model.Expression:
     left = _parse_unary(stream, environment=environment)
     while _PRECEDENCE.get(stream.current_token.token_type, -1) >= min_precedence:
         operator = stream.consume()
@@ -486,7 +510,7 @@ def _parse_variable_declaration(stream: TokenStream) -> model.VariableDeclaratio
     )
 
 
-def _parse_assignment(stream: TokenStream) -> effects.Assignment:
+def _parse_assignment(stream: TokenStream) -> model.Assignment:
     stream.expect("assign")
     name = stream.expect(lexer.TokenType.IDENTIFIER).text
     if stream.check(lexer.TokenType.INTEGER):
@@ -497,7 +521,7 @@ def _parse_assignment(stream: TokenStream) -> effects.Assignment:
         index = 0
     stream.expect(":=")
     value = parse_expression(stream)
-    return effects.Assignment(effects.Name(name), value=value, index=index)
+    return model.Assignment(expressions.Name(name), value=value, index=index)
 
 
 def _parse_location(stream: TokenStream, automaton: model.Automaton) -> model.Location:
@@ -505,7 +529,7 @@ def _parse_location(stream: TokenStream, automaton: model.Automaton) -> model.Lo
     stream.expect("location")
     name = stream.expect(lexer.TokenType.IDENTIFIER).text
     progress_invariant: t.Optional[model.Expression] = None
-    transient_values: t.Set[effects.Assignment] = set()
+    transient_values: t.Set[model.Assignment] = set()
     if stream.accept(":"):
         stream.expect(lexer.TokenType.INDENT)
         while not stream.accept(lexer.TokenType.DEDENT):
@@ -573,7 +597,7 @@ def parse_automaton(stream: TokenStream, ctx: model.Context) -> model.Automaton:
                 elif stream.accept("to"):
                     target_name = stream.expect(lexer.TokenType.IDENTIFIER).text
                     probability: t.Optional[model.Expression] = None
-                    assignments: t.Set[effects.Assignment] = set()
+                    assignments: t.Set[model.Assignment] = set()
                     if stream.accept(":"):
                         stream.expect(lexer.TokenType.INDENT)
                         while not stream.accept(lexer.TokenType.DEDENT):
@@ -660,18 +684,26 @@ def _parse_network(stream: TokenStream, ctx: model.Context) -> None:
                     if not stream.accept(","):
                         stream.expect(")")
                         break
-            input_enable: t.Set[str] = set()
+            input_enable: t.Set[actions.ActionType] = set()
             if stream.accept(":"):
                 stream.expect(lexer.TokenType.INDENT)
                 if stream.accept("input"):
                     stream.expect("enable")
-                    input_enable.add(stream.expect(lexer.TokenType.IDENTIFIER).text)
+                    input_enable.add(
+                        ctx.get_action_type_by_name(
+                            stream.expect(lexer.TokenType.IDENTIFIER).text
+                        )
+                    )
                     while stream.accept(","):
-                        input_enable.add(stream.expect(lexer.TokenType.IDENTIFIER).text)
+                        input_enable.add(
+                            ctx.get_action_type_by_name(
+                                stream.expect(lexer.TokenType.IDENTIFIER).text
+                            )
+                        )
                 stream.expect(lexer.TokenType.DEDENT)
             instance_map[name] = ctx.get_automaton_by_name(
                 automaton_name
-            ).create_instance(parameters=parameters, input_enable=input_enable)
+            ).create_instance(arguments=parameters, input_enable=input_enable)
         elif stream.accept("composition"):
             instances: t.List[model.Instance] = []
             instances.append(
@@ -749,7 +781,7 @@ def _parse_moml(stream: TokenStream, ctx: model.Context) -> model.Context:
             assert automaton.name not in automaton_map and automaton.name is not None
             automaton_map[automaton.name] = automaton
         elif stream.check("action"):
-            ctx.add_action_type(_parse_action_declaration(stream))
+            ctx._add_action_type(_parse_action_declaration(stream))
         elif stream.check("network"):
             _parse_network(stream, ctx)
         elif stream.check(lexer.TokenType.END_OF_FILE):
